/**
 * Namespace for infraction related classes.
 * 
 * @author	Tim Duesterhus, Alexander Ebert
 * @copyright	2001-2013 WoltLab GmbH
 * @license	GNU Lesser General Public License <http://opensource.org/licenses/lgpl-license.php>
 */
WCF.Infraction = { };

/**
 * Namespace for suspension related classes.
 */
WCF.Infraction.Suspension = { };

/**
 * Namespace for warning related classes.
 */
WCF.Infraction.Warning = { };

/**
 * Handles content warning.
 * 
 * @param	string		objectType
 * @param	string		buttonSelector
 */
WCF.Infraction.Warning.Content = Class.extend({
	/**
	 * list of buttons
	 * @var	object
	 */
	_buttons: { },
	
	/**
	 * button selector
	 * @var	string
	 */
	_buttonSelector: '',
	
	/**
	 * dialog overlay
	 * @var	jQuery
	 */
	_dialog: null,
	
	/**
	 * notification object
	 * @var	WCF.System.Notification
	 */
	_notification: null,
	
	/**
	 * object id
	 * @var	integer
	 */
	_objectID: 0,
	
	/**
	 * object type name
	 * @var	string
	 */
	_objectType: '',
	
	/**
	 * user id
	 * @var	integer
	 */
	_userID: 0,
	
	/**
	 * action proxy
	 * @var	WCF.Action.Proxy
	 */
	_proxy: null,
	
	/**
	 * Creates a new WCF.Moderation.Report object.
	 * 
	 * @param	string		objectType
	 * @param	string		buttonSelector
	 */
	init: function(objectType, buttonSelector) {
		this._objectType = objectType;
		this._buttonSelector = buttonSelector;
		
		this._buttons = { };
		this._notification = null;
		this._objectID = 0;
		this._proxy = new WCF.Action.Proxy({
			success: $.proxy(this._success, this)
		});
		
		this._initButtons();
		
		WCF.DOMNodeInsertedHandler.addCallback('WCF.Infraction.Warn' + this._objectType.hashCode(), $.proxy(this._initButtons, this));
	},
	
	/**
	 * Initializes the report feature for all matching buttons.
	 */
	_initButtons: function() {
		var self = this;
		$(this._buttonSelector).each(function(index, button) {
			var $button = $(button);
			var $buttonID = $button.wcfIdentify();
			
			if (!self._buttons[$buttonID]) {
				self._buttons[$buttonID] = $button;
				$button.click($.proxy(self._click, self));
			}
		});
	},
	
	/**
	 * Handles clicks on a report button.
	 * 
	 * @param	object		event
	 */
	_click: function(event) {
		this._objectID = $(event.currentTarget).data('objectID');
		this._userID = $(event.currentTarget).data('userID');
		
		this._proxy.setOption('data', {
			actionName: 'prepareWarning',
			className: 'wcf\\data\\user\\infraction\\warning\\UserInfractionWarningAction',
			parameters: {
				objectID: this._objectID,
				objectType: this._objectType,
				userID: this._userID
			}
		});
		this._proxy.sendRequest();
	},
	
	/**
	 * Handles successful AJAX requests.
	 * 
	 * @param	object		data
	 * @param	string		textStatus
	 * @param	jQuery		jqXHR
	 */
	_success: function(data, textStatus, jqXHR) {
		// object has been successfully reported
		if (data.returnValues.warned) {
			if (this._notification === null) {
				this._notification = new WCF.System.Notification(WCF.Language.get('wcf.infraction.warn.success'));
			}
			
			// show success and close dialog
			this._dialog.wcfDialog('close');
			this._notification.show();
		}
		else if (data.returnValues.template) {
			// display template
			this._showDialog(data.returnValues.template);
			
			if (!data.returnValues.alreadyWarned) {
				// bind event listener for buttons
				this._dialog.find('.jsSubmitWarning').click($.proxy(this._submit, this));
			}
		}
	},
	
	/**
	 * Displays the dialog overlay.
	 * 
	 * @param	string		template
	 */
	_showDialog: function(template) {
		if (this._dialog === null) {
			this._dialog = $('#infractionWarn');
			if (!this._dialog.length) {
				this._dialog = $('<div id="infractionWarn" />').hide().appendTo(document.body);
			}
		}
		
		this._dialog.html(template).wcfDialog({
			title: WCF.Language.get('wcf.infraction.warn')
		}).wcfDialog('render');
	},
	
	/**
	 * Submits a report unless the textarea is empty.
	 */
	_submit: function() {
		$('#infractionWarn .formError').removeClass('formError');
		var $title = '', $points = 0, $expiresNever = 0, $expires = 0, $expiresValue = 0, $expiresType = '';
		var $reason = '';
		
		var $warningID = $('#warningIDSelectContainer input:checked').val();
		if (parseInt($warningID, 10) === 0) {
			$title = $('#warningTitle').val().trim();
			if ($title == '') {
				$('#warningTitle').closest('dl').addClass('formError');
				return;
			}
			$points = $('#warningPoints').val();
			if ($points < 0) {
				$('#warningPoints').closest('dl').addClass('formError');
				return;
			}
			$expiresNever = $('#warningExpiresNever').is(':checked') ? 1 : 0;
			if (!$expiresNever) {
				$expires = $('#warningExpires').val();
				$expiresType = $('#warningExpiresType').val();
				
				if ($expires < 1) {
					$('#warningExpires').closest('dl').addClass('formError');
					return;
				}
				
				$expiresValue = 0;
				if ($expiresType == 'hours') $expiresValue = $expires * 3600;
				else if ($expiresType == 'days') $expiresValue = $expires * 86400;
				else if ($expiresType == 'weeks') $expiresValue = $expires * 86400 * 7;
			}
		}
		$reason = $('#warningReason').val().trim();
		if ($reason == '') {
			$('#warningReason').closest('dl').addClass('formError');
			return;
		}
		
		this._proxy.setOption('data', {
			actionName: 'warn',
			className: 'wcf\\data\\user\\infraction\\warning\\UserInfractionWarningAction',
			parameters: {
				reason: $reason,
				title: $title,
				points: $points,
				expires: $expiresValue,
				warningID: $warningID,
				objectID: this._objectID,
				objectType: this._objectType,
				userID: this._userID
			}
		});
		this._proxy.sendRequest();
	}
});

/**
 * Allows warning a user in his profile.
 * 
 * @param	integer		userID
 * @param	boolean		canWarn
 */
WCF.Infraction.Warning.Profile = Class.extend({
	/**
	 * user id
	 * @var	integer
	 */
	_userID: 0,
	
	/**
	 * instance of WCF.Infraction.Warning.Content
	 * @var WCF.Infraction.Warning.Content
	 */
	_warnHandler: null,
	
	/**
	 * Initializes the WCF.Infraction.Warning.Profile class.
	 * 
	 * @param	integer		userID
	 */
	init: function (userID) {
		this._userID = userID;
		
		$('<li id="warnUser"><a class="button jsTooltip" title="' + WCF.Language.get('wcf.infraction.warn') + '"><span class="icon icon16 icon-legal"></span> <span class="invisible">' + WCF.Language.get('wcf.infraction.warn') + '</span></a></li>').data('userID', this._userID).prependTo($('#profileButtonContainer'));
		this._warnHandler = new WCF.Infraction.Warning.Content('', '#warnUser');
	}
});

/**
 * Revokes a warning.
 */
WCF.Infraction.Warning.Revoke = Class.extend({
	/**
	 * button selector
	 * @var	string
	 */
	_buttonSelector: '',
	
	/**
	 * class name
	 * @var	string
	 */
	_className: '',
	
	/**
	 * confirm message
	 * @var	string
	 */
	_confirmMessage: '',
	
	/**
	 * action proxy
	 * @var	WCF.Action.Proxy
	 */
	_proxy: null,
	
	/**
	 * table row selector
	 * @var	string
	 */
	_rowSelector: '',
	
	/**
	 * success message
	 * @var	string
	 */
	_successMessage: '',
	
	/**
	 * Initializes the WCF.Infraction.Warning.Revoke class.
	 */
	init: function() {
		this._buttonSelector = '.jsButtonWarningRevoke';
		this._className = 'wcf\\data\\user\\infraction\\warning\\UserInfractionWarningAction';
		this._confirmMessage = 'wcf.infraction.warning.revoke.confirmMessage';
		this._rowSelector = '.jsWarningRow';
		this._successMessage = 'wcf.infraction.warning.revoke.success';
		
		this._proxy = new WCF.Action.Proxy({
			success: $.proxy(this._success, this)
		});
		
		this._bindListener();
	},
	
	/**
	 * Binds event listeners.
	 */
	_bindListener: function() {
		$(this._buttonSelector).click($.proxy(this._click, this));
	},
	
	/**
	 * Handles clicks on the revoke button.
	 * 
	 * @param	object		event
	 */
	_click: function(event) {
		var $objectID = $(event.currentTarget).parents(this._rowSelector).data('objectID');
		
		WCF.System.Confirmation.show(WCF.Language.get(this._confirmMessage), $.proxy(function(action) {
			if (action === 'confirm') {
				this._proxy.setOption('data', {
					actionName: 'revoke',
					className: this._className,
					objectIDs: [ $objectID ]
				});
				this._proxy.sendRequest();
			}
		}, this));
	},
	
	/**
	 * Handles successful AJAX requests.
	 * 
	 * @param	object		data
	 * @param	string		textStatus
	 * @param	jQuery		jqXHR
	 */
	_success: function(data, textStatus, jqXHR) {
		$(this._rowSelector).each($.proxy(function(index, row) {
			var $row = $(row);
			if (WCF.inArray($row.data('objectID'), data.objectIDs)) {
				$row.find(this._buttonSelector).addClass('disabled').removeClass('pointer').off('click');
				
				var $notification = new WCF.System.Notification(WCF.Language.get(this._successMessage));
				$notification.show();
			}
		}, this));
	}
});

/**
 * Handles revoking of suspensions.
 * 
 * @see	WCF.Infraction.Warning.Revoke
 */
WCF.Infraction.Suspension.Revoke = WCF.Infraction.Warning.Revoke.extend({
	/**
	 * @see	WCF.Infraction.Warning.Revoke._bindListener()
	 */
	_bindListener: function() {
		this._buttonSelector = '.jsButtonSuspensionRevoke';
		this._className = 'wcf\\data\\user\\infraction\\suspension\\UserInfractionSuspensionAction';
		this._confirmMessage = 'wcf.infraction.suspension.revoke.confirmMessage';
		this._rowSelector = '.jsSuspensionRow';
		this._successMessage = 'wcf.infraction.suspension.revoke.success';
		
		this._super();
	}
});

/**
 * Display details for a specific warning.
 */
WCF.Infraction.Warning.ShowDetails = Class.extend({
	/**
	 * list of cached templates
	 * @var	object
	 */
	_cache: { },
	
	/**
	 * dialog overlay
	 * @var	jQuery
	 */
	_dialog: null,
	
	/**
	 * action proxy
	 * @var	WCF.Action.Proxy
	 */
	_proxy: null,
	
	/**
	 * Initializes the WCF.Infraction.Warning.ShowDetails class.
	 */
	init: function() {
		this._cache = { };
		this._dialog = null;
		this._proxy = new WCF.Action.Proxy({
			success: $.proxy(this._success, this)
		});
		
		$('.jsInfractionShowDetails').click($.proxy(this._click, this));
	},
	
	/**
	 * Handles clicks on the details link.
	 * 
	 * @param	object		event
	 */
	_click: function(event) {
		var $userWarningID = $(event.currentTarget).parents('.jsWarningRow').data('objectID');
		
		if (this._cache[$userWarningID] === undefined) {
			this._proxy.setOption('data', {
				actionName: 'getDetails',
				className: 'wcf\\data\\user\\infraction\\warning\\UserInfractionWarningAction',
				objectIDs: [ $userWarningID ]
			});
			this._proxy.sendRequest();
		}
		else {
			this._showDetails($userWarningID);
		}
		
		event.preventDefault();
		return false;
	},
	
	/**
	 * Displays an overlay for a specific warning id.
	 * 
	 * @param	integer		userWarningID
	 */
	_showDetails: function(userWarningID) {
		if (this._dialog === null) {
			this._dialog = $('<div />').hide().appendTo(document.body);
			this._dialog.html(this._cache[userWarningID]).wcfDialog({
				title: WCF.Language.get('wcf.infraction.warning.details')
			});
		}
		else {
			this._dialog.html(this._cache[userWarningID]).wcfDialog('open');
		}
	},
	
	/**
	 * Handles successful AJAX requests.
	 * 
	 * @param	object		data
	 * @param	string		textStatus
	 * @param	jQuery		jqXHR
	 */
	_success: function(data, textStatus, jqXHR) {
		this._cache[data.returnValues.userWarningID] = data.returnValues.template;
		this._showDetails(data.returnValues.userWarningID);
	}
});

