<?php
namespace wcf\acp\form;
use wcf\data\infraction\warning\InfractionWarningAction;
use wcf\data\infraction\warning\InfractionWarningEditor;
use wcf\data\package\PackageCache;
use wcf\form\AbstractForm;
use wcf\system\exception\UserInputException;
use wcf\system\language\I18nHandler;
use wcf\system\WCF;
use wcf\util\ArrayUtil;
use wcf\util\StringUtil;

/**
 * Shows the warning add form.
 *
 * @author	Tim Duesterhus
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wcf.infraction
 * @subpackage	acp.form
 * @category	Community Framework (commercial)
 */
class WarningAddForm extends AbstractForm {
	/**
	 * @see wcf\page\AbstractPage::$templateName
	 */
	public $templateName = 'warningAdd';
	
	/**
	 * @see wcf\acp\page\AbstractPage::$activeMenuItem
	 */
	public $activeMenuItem = 'wcf.acp.menu.link.infraction.warning.add';
	
	/**
	 * @see wcf\page\AbstractPage::$neededPermissions
	 */
	public $neededPermissions = array('admin.user.infraction.canManageWarning');
	
	/**
	 * @see wcf\page\AbstractPage::$neededModules
	 */
	public $neededModules = array('MODULE_USER_INFRACTION');
	
	/**
	 * title value
	 * @var	string
	 */
	public $title = '';
	
	/**
	 * points value
	 * @var	integer
	 */
	public $points = 0;
	
	/**
	 * expires value
	 * @var	integer
	 */
	public $expires = 1;
	
	/**
	 * expires type
	 * @var	string
	 */
	public $expiresType = 'days';
	
	/**
	 * true if warning never expires
	 * @var	boolean
	 */
	public $expiresNever = 1;
	
	/**
	 * @see wcf\page\IPage::readParameters()
	 */
	public function readParameters() {
		parent::readParameters();
		
		I18nHandler::getInstance()->register('title');
	}
	
	/**
	 * @see wcf\form\IForm::readFormParameters()
	 */
	public function readFormParameters() {
		parent::readFormParameters();
		
		I18nHandler::getInstance()->readValues();
		
		if (I18nHandler::getInstance()->isPlainValue('title')) $this->title = I18nHandler::getInstance()->getValue('title');
		if (isset($_POST['points'])) $this->points = intval($_POST['points']);
		
		$this->expiresNever = 0;
		if (isset($_POST['expiresNever'])) $this->expiresNever = 1;
		else {
			if (isset($_POST['expires'])) $this->expires = intval($_POST['expires']);
			if (isset($_POST['expiresType'])) $this->expiresType = $_POST['expiresType'];
		}
	}
	
	/**
	 * @see wcf\form\IForm::validate()
	 */
	public function validate() {
		parent::validate();
		
		// validate title
		if (!I18nHandler::getInstance()->validateValue('title')) {
			throw new UserInputException('title');
		}
		
		if ($this->points < 0 || $this->points > 8388607) {
			throw new UserInputException('points', 'invalid');
		}
		
		if ($this->expires < 1) {
			throw new UserInputException('expires', 'invalid');
		}
		if ($this->expiresType != 'hours' && $this->expiresType != 'days' && $this->expiresType != 'weeks')  {
			throw new UserInputException('expires', 'invalid');
		}
	}
	
	/**
	 * @see wcf\form\IForm::save()
	 */
	public function save() {
		parent::save();
		
		$expires = 0;
		if (!$this->expiresNever) {
			switch ($this->expiresType) {
				case 'hours':
					$expires = $this->expires * 3600;
					break;
				case 'days':
					$expires = $this->expires * 86400;
					break;
				case 'weeks':
					$expires = $this->expires * 86400 * 7;
					break;
			}
		}
		
		// save warning
		$this->objectAction = new InfractionWarningAction(array(), 'create', array('data' => array(
			'title' => $this->title,
			'points' => $this->points,
			'expires' => $expires
		)));
		$this->objectAction->executeAction();
		$returnValues = $this->objectAction->getReturnValues();
		$warningEditor = new InfractionWarningEditor($returnValues['returnValues']);
		$warningID = $returnValues['returnValues']->warningID;
		
		$packageID = PackageCache::getInstance()->getPackageID('com.woltlab.wcf.infraction');
		
		if (!I18nHandler::getInstance()->isPlainValue('title')) {
			I18nHandler::getInstance()->save('title', 'wcf.infraction.warning.title'.$warningID, 'wcf.infraction.warning', $packageID);
			
			// update title
			$warningEditor->update(array(
				'title' => 'wcf.infraction.warning.title'.$warningID
			));
		}
		
		$this->saved();
		
		// reset values
		$this->title = '';
		$this->points = 0;
		$this->expires = 1;
		$this->expiresType = 'days';
		$this->expiresNever = 1;
		
		I18nHandler::getInstance()->reset();
		
		// show success
		WCF::getTPL()->assign(array(
			'success' => true
		));
	}
	
	/**
	 * @see wcf\page\IPage::assignVariables()
	 */
	public function assignVariables() {
		parent::assignVariables();
		
		I18nHandler::getInstance()->assignVariables();
		
		WCF::getTPL()->assign(array(
			'action' => 'add',
			'title' => $this->title,
			'points' => $this->points,
			'expires' => $this->expires,
			'expiresType' => $this->expiresType,
			'expiresNever' => $this->expiresNever
		));
	}
}
