<?php
namespace wcf\data\user\infraction\warning;
use wcf\data\conversation\ConversationAction;
use wcf\data\infraction\suspension\InfractionSuspensionList;
use wcf\data\infraction\warning\InfractionWarning;
use wcf\data\infraction\warning\InfractionWarningList;
use wcf\data\object\type\ObjectTypeCache;
use wcf\data\user\infraction\suspension\UserInfractionSuspensionAction;
use wcf\data\user\User;
use wcf\data\user\UserProfile;
use wcf\data\AbstractDatabaseObjectAction;
use wcf\data\DatabaseObjectEditor;
use wcf\system\database\util\PreparedStatementConditionBuilder;
use wcf\system\exception\IllegalLinkException;
use wcf\system\exception\PermissionDeniedException;
use wcf\system\exception\SystemException;
use wcf\system\exception\UserInputException;
use wcf\system\exception\ValidateActionException;
use wcf\system\infraction\warning\IWarnableObject;
use wcf\system\WCF;
use wcf\util\ArrayUtil;
use wcf\util\StringUtil;

/**
 * Executes actions on an user infraction warnings.
 *
 * @author	Tim Duesterhus
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wcf.infraction
 * @subpackage	data.user.infraction.warning
 * @category	Community Framework (commercial)
 */
class UserInfractionWarningAction extends AbstractDatabaseObjectAction {
	/**
	 * @see	\wcf\data\AbstractDatabaseObjectAction::$className
	 */
	protected $className = 'wcf\data\user\infraction\warning\UserInfractionWarningEditor';
	
	/**
	 * user object
	 * @var	wcf\data\user\User
	 */
	public $user = null;
	
	/**
	 * user infraction warning editor object
	 * @var	wcf\data\user\infraction\warning\UserInfractionWarningEditor
	 */
	public $warningEditor = null;
	
	/**
	 * @see	wcf\data\AbstractDatabaseObjectAction\AbstractDatabaseObjectAction::create();
	 */
	public function create() {
		$userWarning = parent::create();
		
		// create conversation
		$userLanguage = $userWarning->getUser()->getLanguage();
		$conversationAction = new ConversationAction(array(), 'create', array(
			'data' => array(
				'subject' => $userLanguage->getDynamicVariable('wcf.infraction.userWarning.message.subject', array('warning' => $userWarning)),
				'time' => TIME_NOW,
				'userID' => $userWarning->judgeID,
				'username' => $userWarning->getJudge()->username
			),
			'messageData' => array(
				'message' => $userLanguage->getDynamicVariable('wcf.infraction.userWarning.message.text', array('warning' => $userWarning))
			),
			'participants' => array(
				$userWarning->userID
			)
		));
		$returnValues = $conversationAction->executeAction();
		// hide conversation
		$conversationAction = new ConversationAction(array($returnValues['returnValues']), 'hideConversation', array('hideConversation' => \wcf\data\conversation\Conversation::STATE_HIDDEN));
		$conversationAction->executeAction();
		
		// check suspensions
		$conditionBuilder = new PreparedStatementConditionBuilder();
		$conditionBuilder->add('userID = ?', array($this->parameters['data']['userID']));
		$conditionBuilder->add('(expires >= ? OR expires = ?)', array(TIME_NOW, 0));
		$conditionBuilder->add('revoked = ?', array(0));
		
		// calculate sum of warning points
		$sql = "SELECT	COALESCE(SUM(points) , 0) AS points
			FROM	wcf".WCF_N."_user_infraction_warning
			".$conditionBuilder;
		$statement = WCF::getDB()->prepareStatement($sql);
		$statement->execute($conditionBuilder->getParameters());
		$row = $statement->fetchArray();
		$points = $row['points'];
		
		// fetch active suspensions
		$sql = "SELECT	suspensionID
			FROM	wcf".WCF_N."_user_infraction_suspension
			".$conditionBuilder;
		$statement = WCF::getDB()->prepareStatement($sql);
		$statement->execute($conditionBuilder->getParameters());
		$activeSuspensions = array();
		while ($row = $statement->fetchArray()) {
			$activeSuspensions[] = $row['suspensionID'];
		}
		
		// fetch possible actions
		$objectTypes = ObjectTypeCache::getInstance()->getObjectTypes('com.woltlab.wcf.infraction.suspensionAction');
		$objectTypeIDs = array();
		foreach ($objectTypes as $objectType) {
			$objectTypeIDs[] = $objectType->objectTypeID;
		}
		
		$suspensionList = new InfractionSuspensionList();
		$suspensionList->getConditionBuilder()->add('objectTypeID IN (?)', array($objectTypeIDs));
		$suspensionList->getConditionBuilder()->add('points <= ?', array($points));
		if (!empty($activeSuspensions)) $suspensionList->getConditionBuilder()->add('suspensionID NOT IN (?)', array($activeSuspensions));
		$suspensionList->readObjects();
		
		// apply suspensions
		foreach ($suspensionList as $suspension) {
			$suspensionAction = new UserInfractionSuspensionAction(array(), 'create', array(
				'data' => array(
					'userID' => $this->parameters['data']['userID'],
					'suspensionID' => $suspension->suspensionID,
					'time' => TIME_NOW,
					'expires' => $suspension->expires ? TIME_NOW + $suspension->expires : 0,
					'warningID' => $userWarning->userWarningID
				)
			));
			$suspensionAction->executeAction();
			$return = $suspensionAction->getReturnValues();
			$userSuspension = $return['returnValues'];
			
			$suspension->getAction()->suspend($suspension, $userSuspension);
		}
		
		return $userWarning;
	}
	
	/**
	 * Validates parameters to prepare warning.
	 */
	public function validatePrepareWarning() {
		if (!WCF::getSession()->getPermission('mod.infraction.warning.canWarn') && !WCF::getSession()->getPermission('admin.user.infraction.canManageWarning')) {
			throw new PermissionDeniedException();
		}
		
		$this->readInteger('objectID', true);
		$this->readString('objectType', true);
		$this->readInteger('userID');
		
		$this->parameters['user'] = new UserProfile(new User($this->parameters['userID']));
		if (!$this->parameters['user']->userID) {
			throw new UserInputException('userID');
		}
		else if ($this->parameters['user']->getPermission('mod.infraction.warning.immune')) {
			throw new PermissionDeniedException();
		}
		
		$this->parameters['objectTypeName'] = $this->parameters['objectType'];
		$this->parameters['object'] = null;
		
		if ($this->parameters['objectTypeName']) {
			$this->parameters['objectType'] = ObjectTypeCache::getInstance()->getObjectTypeByName('com.woltlab.wcf.infraction.warnable', $this->parameters['objectTypeName']);
			if ($this->parameters['objectType'] === null) {
				throw new UserInputException('objectType');
			}
			
			$provider = $this->parameters['objectType']->getProcessor();
			$this->parameters['object'] = $provider->getObjectByID($this->parameters['objectID']);
			
			// validate object
			if (!($this->parameters['object'] instanceof IWarnableObject)) {
				throw new SystemException("class '".get_class($this->parameters['object'])."' does not implement 'wcf\system\infraction\warning\IWarnableObject'");
			}
			else if (!$this->parameters['object']->getObjectID()) {
				throw new IllegalLinkException();
			}
			
			// validate ownership
			if ($this->parameters['user']->userID != $this->parameters['object']->getUserID()) {
				throw new UserInputException('objectID');
			}
		}
	}
	
	/**
	 * Prepares warning.
	 */
	public function prepareWarning() {
		if ($this->parameters['objectTypeName']) {
			$warning = UserInfractionWarning::getWarningByUserAndObject($this->parameters['user']->getDecoratedObject(), $this->parameters['objectType'], $this->parameters['object']);
			// content was already reported
			$alreadyWarned = $warning->userWarningID;
		}
		else {
			$alreadyWarned = false;
		}
		
		$warnings = new InfractionWarningList();
		$warnings->readObjects();
		
		WCF::getTPL()->assign(array(
			'alreadyWarned' => $alreadyWarned,
			'user' => $this->parameters['user'],
			'object' => $this->parameters['object'],
			'objectType' => $this->parameters['objectType'],
			'warnings' => $warnings
		));
		
		return array(
			'alreadyWarned' => $alreadyWarned,
			'template' => WCF::getTPL()->fetch('userWarnDialog')
		);
	}
	
	/**
	 * Validates warning.
	 */
	public function validateWarn() {
		$this->validatePrepareWarning();
		
		$this->readInteger('warningID', true);
		
		if ($this->parameters['warningID']) {
			$this->parameters['warning'] = new InfractionWarning($this->parameters['warningID']);
			$this->parameters['title'] = $this->parameters['warning']->title;
			$this->parameters['points'] = $this->parameters['warning']->points;
			$this->parameters['expires'] = $this->parameters['warning']->expires;
		}
		else {
			$this->readString('title');
			$this->readInteger('points', true);
			$this->readInteger('expires', true);
		}
		$this->readString('reason');
		
		// warning
		if ($this->parameters['warningID'] && !$this->parameters['warning']->warningID) {
			throw new UserInputException('warning', 'invalid');
		}
		if (!$this->parameters['warningID'] && !WCF::getSession()->getPermission('mod.infraction.warning.canUseCustomWarning')) {
			throw new UserInputException('warning', 'invalid');
		}
		// points
		if ($this->parameters['points'] < 0) {
			throw new UserInputException('points', 'invalid');
		}
		
		// expires
		if ($this->parameters['expires'] < 0) {
			throw new UserInputException('expires', 'invalid');
		}
	}
	
	/**
	 * Warns a user.
	 */
	public function warn() {
		$objectAction = new self(array(), 'create', array(
			'data' => array(
				'objectID' => $this->parameters['objectID'] ?: null,
				'objectTypeID' => $this->parameters['objectType'] ? $this->parameters['objectType']->objectTypeID : null,
				'userID' =>$this->parameters['user']->userID,
				'judgeID' => WCF::getUser()->userID,
				'warningID' => $this->parameters['warningID'] ?: null,
				'time' => TIME_NOW,
				'title' => $this->parameters['title'],
				'points' => $this->parameters['points'],
				'expires' => $this->parameters['expires'] > 0 ? TIME_NOW + $this->parameters['expires'] : 0,
				'reason' => $this->parameters['reason']
			)
		));
		$objectAction->executeAction();
		
		return array(
			'warned' => 1
		);
	}
	
	/**
	 * Checks permissions for revoking warnings.
	 */
	public function validateRevoke() {
		WCF::getSession()->checkPermissions(array('mod.infraction.warning.canRevoke'));
		
		unset($this->parameters['revoker']);
	}
	
	/**
	 * Revokes warnings.
	 */
	public function revoke() {
		// revoker cannot be set during an AJAX request but will be 'null' within the cronjob
		if (!isset($this->parameters['revoker'])) {
			$this->parameters['revoker'] = WCF::getUser()->userID;
		}
		
		$objectAction = new self($this->objectIDs, 'update', array(
			'data' => array(
				'revoked' => 1,
				'revoker' => $this->parameters['revoker']
			)
		));
		$objectAction->executeAction();
	}
	
	/**
	 * Validates parameters to show details of a warning.
	 */
	public function validateGetDetails() {
		WCF::getSession()->checkPermissions(array('mod.infraction.warning.canWarn'));
		
		$this->warningEditor = $this->getSingleObject();
		if (!$this->warningEditor->userWarningID) {
			throw new UserInputException('objectIDs');
		}
	}
	
	/**
	 * Returns details of a warning.
	 * 
	 * @return	array
	 */
	public function getDetails() {
		WCF::getTPL()->assign(array(
			'warning' => $this->warningEditor
		));
		
		return array(
			'template' => WCF::getTPL()->fetch('userWarningDetails'),
			'userWarningID' => $this->warningEditor->userWarningID
		);
	}
}
